/*****************************************************
Project : OSC1 
Date    : 08.06.2009 
Author  : Sergey Kozlov                        

Chip type           : ATmega8515
Program type        : Application
Clock frequency     : 14,745600 MHz
*****************************************************/

#include <mega8515.h>
#include <delay.h>
 
#define RXB8 1
#define TXB8 0
#define UPE 2
#define OVR 3
#define FE 4
#define UDRE 5
#define RXC 7

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<OVR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)
//void send_string(flash char *string, char len);
void counter_reset(void);
void read_memory(void);
                       
char set_trigger=0;
char send_pc=0;

// External Interrupt 1 service routine
interrupt [EXT_INT1] void ext_int1_isr(void)
{
send_pc=1;
}

void counter_reset(void)
{
        PORTD.5=1;
        delay_ms(1);
        PORTD.5=0; 
}

void read_memory(void)
{
        unsigned int i;
        unsigned int max;
        if(PINB.0) max=1023; else max=16383;//32767;
        PORTD.7=1;//DATA READ 
        for(i=0;i<=max;i++)//32767
                {
                PORTD.4=0; 
                PORTD.4=1;//MCU CLK
                while ( !( UCSRA & (1<<UDRE)) );
                UDR=PINA;
                }
        if(PINB.0) goto end_read; 
        delay_ms(50);
        for(i=0;i<=max;i++)//32767
                {
                PORTD.4=0; 
                PORTD.4=1;//MCU CLK
                while ( !( UCSRA & (1<<UDRE)) );
                UDR=PINA;
                }
        end_read:                    
        PORTD.7=0;
}

void decode_command(char cmd)
{
        char mask;
        char data;        
        
        //if(cmd==0)//device present
        //        {
        //        send_string("Digital Storage Oscilloscope\n\r",30);
        //        return;
        //        }
        mask=cmd&0b11110000;
        if(mask==0b10000000)//set prescaler
                {
                data=cmd&0b00000111;
                PORTB=(data<<5)|(PORTB&0b00011111);
                return;
                }
        mask=cmd&0b11111100;
        if(mask==252)//set trigger source
                {
                mask=0b00000011;
                data=cmd&mask;
                switch(data)
                        {
                        case 0:
                                PORTB=(0b00000111)|(PORTB&0b11100111);
                                set_trigger=1;
                                break;//loop internal
                        case 1:
                                PORTB=(0b00010111)|(PORTB&0b11100111);
                                set_trigger=1;
                                break;//level comp
                        case 2:
                                PORTB=(0b00000111)|(PORTB&0b11100111);
                                set_trigger=1;
                                break;//single internal
                        case 3:
                                PORTB=(0b00011111)|(PORTB&0b11100111);
                                set_trigger=1;
                                break;//external trigger
                        }
                return;
                }
        if(cmd==251)//start conversion if cmd=251
                { 
                set_trigger=1;
                //PORTB.2=0;
                //PORTB.2=1;//transmitt set signal via MUX to trigger
                }
                
}

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
char status,data;
status=UCSRA;
data=UDR;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
   decode_command(data);
   };
}

/*void send_string(flash char *string, char len)
{
        char i;
        #asm("cli");
        for(i=0;i<=len;i++)
                {
                while ( !( UCSRA & (1<<UDRE)) );//wait when data will be sent
                UDR=string[i];
                }
        #asm("sei");
}*/

// Declare your global variables here

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTA=0x00;
DDRA=0x00;

// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=P 
PORTB=0b00000101;//0x01;
//      76543210
DDRB=0b11111100;//0x00;
//     76543210
// Port C initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=In Func2=In Func1=In Func0=In 
// State7=1 State6=0 State5=1 State4=0 State3=P State2=T State1=T State0=T 
PORTD=0xA8;
DDRD=0xF0;
//PORTD.3=0;
// Port E initialization
// Func2=In Func1=In Func0=In 
// State2=T State1=T State0=T 
PORTE=0x00;
DDRE=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: On
// INT1 Mode: Falling Edge
// INT2: Off
GICR|=0x80;
MCUCR=0x08;
EMCUCR=0x00;
GIFR=0x80;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 57600
UCSRA=0x00;
UCSRB=0x98;
UCSRC=0x86;
UBRRH=0x00;
//UBRRL=0x0C;//57600baud
UBRRL=0x07;//115200baud

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
                             
PORTD.7=0;//write mode
counter_reset();
PORTD.6=0;
// Global enable interrupts
#asm("sei")
while (1)
      {
      if(set_trigger && (send_pc==0)) 
        {
        set_trigger=0;
        PORTD.6=0;
        PORTD.6=1;//activate trigger
        PORTD.6=0;
        
        PORTB.2=0;
        PORTB.2=1;//transmitt set signal via MUX to trigger
        };
      if(send_pc) 
        {
        GICR=0x00;//deactivate INT1
        delay_ms(100);
        counter_reset();
        delay_ms(1);
        read_memory();
        delay_ms(1);
        counter_reset();
        delay_ms(10);
        send_pc=0;
        GIFR|=0x80;//clear INT1 flag
        GICR=0x80;//activate INT1
        };
      };
}
